#include "vector.h"

namespace m2if {

Vector::Vector(int dim) : Matrix(dim, 1)
{
}

Vector::Vector(const Vector & v) : Matrix(v)
{
}

double & Vector::operator[](int i)
{
  if ((i < 0) || (i >= nr_))
    throw "wrong index";
  return operator()(i, 0);
}

double Vector::operator[](int i) const
{
  if ((i < 0) || (i >= nr_))
    throw "wrong index";
  return operator()(i, 0);
}

Vector operator+(const Vector & v1, const Vector & v2)
{
  Matrix m = (Matrix)v1 + (Matrix)v2;
  Vector v(v1.get_dim());
  v.copy_data(m);

  return v;
}

Vector operator-(const Vector & v1, const Vector & v2)
{
  Matrix m = (Matrix)v1 - (Matrix)v2;
  Vector v(v1.get_dim());
  v.copy_data(m);

  return v;
}

double operator*(const Vector & v1, const Vector & v2)
{
  if (v1.get_dim() != v2.get_dim())
    throw "wrong size";

  double res = 0;
  for (int i = 0; i < v1.get_dim(); i++)
    res += v1[i] * v2[i];

  return res;
}

Vector operator*(const Matrix & m, const Vector & v)
{
  Matrix tmp = m * static_cast<Matrix>(v);
  Vector res(m.get_nr());
  res.copy_data(tmp);

  return res;
}

void Vector::copy_data(const Matrix & m)
{
  for (int i = 0; i < nr_; i++)
  {
    data_[i][0] = m(i, 0);
  }
}

} // namespace m2if
